import depoc
import click
import time
import sys

from typing import Any

from .utils._response import _handle_response
from .utils._format import _format_response, spinner


client = depoc.DepocClient()


@click.group
def supplier() -> None:
    ''' Manage suppliers. '''

@supplier.command
def create() -> None:
    ''' Create supplier. '''
    click.echo(f'\n{'ADD NEW SUPPLIER':-<50}')

    data: dict[str, Any] = {}
    data.update({'code': input('Supplier Internal Code: ')})
    data.update({'legal_name': input('Legal Name: ')})
    data.update({'trade_name': input('Trade Name: ')})
    data.update({'cnpj': input('CNPJ: ')})
    data.update({'ie': input('IE: ')})
    data.update({'im': input('IM: ')})
    data.update({'notes': input('Notes: ')})
    data.update({'phone': input('Phone: ')})
    data.update({'email': input('Email: ')})
    data.update({'postcode': input('Postal Code: ')})
    data.update({'city': input('City: ')})
    data.update({'state': input('State: ')})
    data.update({'address': input('Address: ')})

    click.echo(f'{'':-<50}')

    service = client.suppliers.create

    if obj := _handle_response(service, data):
        title = obj.legal_name
        header = f'{obj.id}\n'
        highlight = obj.trade_name
        _format_response(obj, title, header, highlight)

@supplier.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific supplier. '''
    service = client.suppliers.get

    if obj := _handle_response(service, resource_id=id):
        title = obj.legal_name
        header = f'{obj.id}\n'
        highlight = obj.trade_name
        _format_response(obj, title, header, highlight)

@supplier.command
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
@click.option('-d', '--detailed', is_flag=True)
def all(limit: int, page: int, detailed: bool) -> None:
    ''' Retrieve all customers. '''
    service = client.suppliers.all

    if response := _handle_response(service, limit=limit, page=page):
        click.echo(f'\nResults: {response.count}')
        if limit < response.count:
            click.echo(
                f'Showing: {len(response.results)} out of {response.count}'
            ) 
        if response.next:
            click.echo(f'For next page: --page <number>')

        for obj in response.results:
            title = obj.legal_name
            header = f'{obj.id}'
            highlight = obj.trade_name

            remove = [] if detailed else \
            [item for item in obj.to_dict().keys() if item != '-']

            _format_response(obj, title, header, highlight, remove=remove)

@supplier.command
@click.argument('id')
@click.option('-c', '--code')
@click.option('-ln', '--legal_name')
@click.option('-tn', '--trade_name')
@click.option('-p', '--phone')
@click.option('-e', '--email')
@click.option('-p', '--postcode')
@click.option('-s', '--state')
@click.option('--cnpj')
@click.option('--ie')
@click.option('--im')
@click.option('--notes')
@click.option('--city')
@click.option('--address')
def update(
    id: str,
    code: str,
    legal_name: str,
    trade_name: str,
    notes: str,
    phone: str,
    email: str,
    postcode: str,
    city: str,
    state: str,
    address: str,
    cnpj: str,
    ie: str,
    im: str,
    ) -> None:
    ''' Update an specific supplier. '''
    data: dict[str, Any] = {}
    data.update({'code': code}) if code else None
    data.update({'legal_name': legal_name}) if legal_name else None
    data.update({'trade_name': trade_name}) if trade_name else None
    data.update({'cnpj': cnpj}) if cnpj else None
    data.update({'ie': ie}) if ie else None
    data.update({'im': im}) if im else None
    data.update({'notes': notes}) if notes else None
    data.update({'phone': phone}) if phone else None
    data.update({'email': email}) if email else None
    data.update({'postcode': postcode}) if postcode else None
    data.update({'city': city}) if city else None
    data.update({'state': state}) if state else None
    data.update({'address': address}) if address else None

    service = client.suppliers.update

    if obj := _handle_response(service, data, resource_id=id):
        header = f'{obj.id}\n'
        highlight = obj.trade_name
        _format_response(obj, 'UPDATED', header, highlight, color='green')

@supplier.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete an specific supplier. '''
    service = client.suppliers.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if obj := _handle_response(service, resource_id=id):
            _format_response(obj, 'DELETED', 'Done', color='red')



import click
import sys
import itertools
import time

from typing import Literal

from depoc.objects.base import DepocObject


def _format_response(
        obj: DepocObject,
        title: str,
        header: str,
        highlight: str | None = None,
        color: Literal[
            'red',
            'green',
            'yellow',
            'blue',
            'magenta',
            'cyan',
        ] = 'yellow',
        remove: list[str] | None = None,
    ):
    
    try:
        if obj.is_active == False:
            color = 'red'
    except AttributeError:
        pass

    title = click.style(f'{title.upper():-<50}', fg=color, bold=True)
    header = click.style(f'\n{header:>50}', bold=True)

    if highlight:
        if len(highlight) > 50:
            highlight = highlight[:50] if len(highlight) > 50 else None
        highlight = click.style(f'\n{highlight:>50}', bold=True)

    data = obj.to_dict()
    body: str = ''

    if remove:
        for item in remove:
            data.pop(item)

    for k, v in data.items():
        k = k.replace('_', ' ').title()
        k = k.upper() if k == 'Id' else k

        if isinstance(v, DepocObject):
            if hasattr(v, 'name'):
                v = v.name

        body += f'\n{k}: {v}'

    response = (
        f'{title}'
        f'{header}'
        f'{highlight if highlight else ''}'
        f'{body}'
    )
    click.echo(response)


def spinner() -> None:
    spinner_cycle = itertools.cycle(['-', '\\', '|', '/'])
    for _ in range(20):
        sys.stdout.write(f'\rDeleting {next(spinner_cycle)} ')
        sys.stdout.flush()
        time.sleep(0.1)
    click.echo('')



import depoc
import requests

from typing import Literal, Any

from depoc.utils import _handle_response


class Requestor(object):
    def __init__(self):
        self._session = requests.Session()
        self._session.headers.update({
            'User-Agent': 'depoc/0.0.3 (Python)',
            'Content-Type': 'application/json',
        })

    def request(
        self,
        method: Literal['GET', 'POST', 'PATCH', 'PUT', 'DELETE'],
        endpoint: str,
        params: dict[str, Any] | None = None,
    ) -> dict[str, Any]:
        url = f'{depoc.BASE_URL}/{endpoint}'
        auth = {'Authorization': f'Bearer {depoc.token}'}
        
        try:
            response = self._session.request(
                method,
                url,
                json=params,
                headers=auth,
            )
            return _handle_response(response)

        except requests.exceptions.RequestException as e:
            raise Exception(str(e)) from e
        except requests.exceptions.HTTPError as e:
            raise Exception(str(e)) from e



import functools

from time import sleep
from requests.exceptions import RequestException


def retry(retries=3, delay=1):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            for attempt in range(retries):
                try:
                    return func(*args, **kwargs)
                except (RequestException) as e:
                    if attempt == retries - 1:
                        raise
                    sleep(delay * (2 ** attempt))
            return func(*args, **kwargs)
        return wrapper
    return decorator



from typing import Any

from .base import T, APIResource


class Finder(APIResource[T]):
    @classmethod
    def filter(
            cls,
            search: str | None = None,
            date: str | None = None,
            start_date: str | None = None,
            end_date: str | None = None,
            limit: int | None = None,
        ):
        endpoint = f'{cls.endpoint}/?'

        if search:
            endpoint += f'&search={search}'
        
        if date:
            endpoint += f'&date={date}'

        if start_date and end_date:
            endpoint += f'&start_date={start_date}&end_date={end_date}'

        response = cls.requestor.request('GET', endpoint)
        return cls._paginate(response, limit)


class Create(APIResource[T]):
    @classmethod
    def create(cls, params: dict[str, Any], resource_id: str | None = None) -> T:
        endpoint = cls.endpoint
        if '<id>' in endpoint and resource_id:
            endpoint = cls.endpoint.replace('<id>', resource_id)

        response = cls.requestor.request('POST', endpoint, params)
        return cls._convert_to_object(response)


class Retrieve(APIResource[T]):
    @classmethod
    def get(
            cls, 
            resource_id: str | None = None,
            resource_id2: str | None = None,
        ) -> T:
        endpoint = cls.endpoint

        endpoint = f'{endpoint}/{resource_id}' if resource_id else endpoint

        if '<id>' in endpoint and resource_id:
            endpoint = cls.endpoint.replace('<id>', resource_id)
            if resource_id2:
                endpoint += f'/{resource_id2}'

        response = cls.requestor.request('GET', endpoint)
        return cls._convert_to_object(response)
    
    @classmethod
    def all(
        cls,
        resource_id: str | None = None,
        *,
        limit: int | None = None, 
        page: int | None = None,
    ) -> T:
        endpoint = cls.endpoint
        if '<id>' in endpoint and resource_id:
            endpoint = cls.endpoint.replace('<id>', resource_id)
            
        endpoint = f'{endpoint}?page={page}' if page else endpoint
        response = cls.requestor.request('GET', endpoint)
        return cls._paginate(response, limit)


class Update(APIResource[T]):
    @classmethod
    def update(
        cls,
        params: dict[str, Any],
        resource_id: str | None = None,
        resource_id2: str | None = None,
    ) -> T:
        endpoint = cls.endpoint

        endpoint = f'{endpoint}/{resource_id}' if resource_id else endpoint

        if '<id>' in endpoint and resource_id:
            endpoint = cls.endpoint.replace('<id>', resource_id)
            if resource_id2:
                endpoint += f'/{resource_id2}'

        response = cls.requestor.request('PATCH', endpoint, params)
        return cls._convert_to_object(response)


class Delete(APIResource[T]):
    @classmethod
    def delete(
            cls,
            resource_id: str | None = None,
            resource_id2: str | None = None,
        ) -> T:
        endpoint = cls.endpoint

        endpoint = f'{endpoint}/{resource_id}' if resource_id else endpoint

        if '<id>' in endpoint and resource_id and resource_id2:
            endpoint = cls.endpoint.replace('<id>', resource_id)
            endpoint += f'/{resource_id2}'

        response = cls.requestor.request('DELETE', endpoint)
        return cls._convert_to_object(response)



import depoc
import click

from .utils._response import _handle_response
from .utils._format import _format_response


client = depoc.DepocClient()


@click.command
def me() -> None:
    ''' Get the current user's data'''
    service = client.me.get

    if obj := _handle_response(service):
        title = f'Welcome {obj.name}'
        header = f'@{obj.username}'
        _format_response(obj, title, header, obj.email)



import click

from .commands import account
from .commands import login
from .commands import logout
from .commands import me
from .commands import finance
from .commands import contact
from .commands import receivable
from .commands import payable
from .commands import report
from .commands import customer
from .commands import supplier
from .commands import owner
from .commands import balance

@click.group()
def main() -> None:
   pass
 
main.add_command(account)
main.add_command(login)
main.add_command(logout)
main.add_command(me)
main.add_command(finance.bank)
main.add_command(finance.category)
main.add_command(finance.transaction)
main.add_command(contact)
main.add_command(receivable)
main.add_command(payable)
main.add_command(report)
main.add_command(customer)
main.add_command(supplier)
main.add_command(owner)
main.add_command(balance)



import depoc
import click
import time
import sys

from typing import Any

from .utils._response import _handle_response
from .utils._format import _format_response, spinner


client = depoc.DepocClient()


@click.group
def customer() -> None:
    ''' Manage customers. '''

@customer.command
def create() -> None:
    ''' Create customer. '''

    click.echo(f'\n{'ADD NEW CUSTOMER':-<50}')

    data: dict[str, Any] = {}
    data.update({'code': input('Customer Internal Code: ')})
    data.update({'name': input('Name: ')})
    data.update({'alias': input('Customer Alias: ')})
    data.update({'gender': input('Gender: ').lower()})
    data.update({'cpf': input('CPF: ')})
    data.update({'notes': input('Notes: ')})
    data.update({'phone': input('Phone: ')})
    data.update({'email': input('Email: ')})
    data.update({'postcode': input('Postal Code: ')})
    data.update({'city': input('City: ')})
    data.update({'state': input('State: ')})
    data.update({'address': input('Address: ')})

    click.echo(f'{'':-<50}')

    service = client.customers.create

    if obj := _handle_response(service, data):
        title = obj.name
        header = f'{obj.id}\n'
        highlight = obj.alias
        _format_response(obj, title, header, highlight)

@customer.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific customer. '''
    service = client.customers.get

    if obj := _handle_response(service, resource_id=id):
        title = obj.name
        header = f'{obj.id}\n'
        highlight = obj.alias
        _format_response(obj, title, header, highlight)

@customer.command
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
@click.option('-d', '--detailed', is_flag=True)
def all(limit: int, page: int, detailed: bool) -> None:
    ''' Retrieve all customers. '''
    service = client.customers.all

    if response := _handle_response(service, limit=limit, page=page):
        click.echo(f'\nResults: {response.count}')
        if limit < response.count:
            click.echo(
                f'Showing: {len(response.results)} out of {response.count}'
            ) 
        if response.next:
            click.echo(f'For next page: --page <number>')

        for obj in response.results:
            title = obj.name
            header = f'{obj.id}\n'
            highlight = obj.alias

            remove = [] if detailed else \
            [item for item in obj.to_dict().keys() if item != '-']

            _format_response(obj, title, header, highlight, remove=remove)

@customer.command
@click.argument('id')
@click.option('-c', '--code')
@click.option('-n', '--name')
@click.option('-a', '--alias')
@click.option('-g', '--gender')
@click.option('-p', '--phone')
@click.option('-e', '--email')
@click.option('-p', '--postcode')
@click.option('-s', '--state')
@click.option('--cpf')
@click.option('--notes')
@click.option('--city')
@click.option('--address')
def update(
    id: str,
    code: str,
    name: str,
    alias: str,
    gender: str,
    cpf: str,
    notes: str,
    phone: str,
    email: str,
    postcode: str,
    city: str,
    state: str,
    address: str,
    ) -> None:
    ''' Update an specific customer. '''
    data: dict[str, Any] = {}
    data.update({'code': code}) if code else None
    data.update({'name': name}) if name else None
    data.update({'alias': alias}) if alias else None
    data.update({'gender': gender}) if gender else None
    data.update({'cpf': cpf}) if cpf else None
    data.update({'notes': notes}) if notes else None
    data.update({'phone': phone}) if phone else None
    data.update({'email': email}) if email else None
    data.update({'postcode': postcode}) if postcode else None
    data.update({'city': city}) if city else None
    data.update({'state': state}) if state else None
    data.update({'address': address}) if address else None

    service = client.customers.update

    if obj := _handle_response(service, data, resource_id=id):
        header = f'{obj.id}\n'
        highlight = obj.alias
        _format_response(obj, 'UPDATED', header, highlight, color='green')

@customer.command
@click.argument('ids', nargs=-1)
def delete(ids: str) -> None:
    ''' Delete an specific customer. '''
    service = client.customers.delete

    while True:
        prompt = click.style('Proceed to deletion? [y/n] ', fg='red')
        confirmation = input(prompt)
        if confirmation == 'n':
            sys.exit(0)
        elif confirmation == 'y':
            break

    if len(ids) > 1:
        spinner()

    for id in ids:
        time.sleep(0.5)
        if obj := _handle_response(service, resource_id=id):
            _format_response(obj, 'DELETED', 'Done', color='red')



import depoc
import click

from .utils._response import _handle_response
from .utils._format import _format_response


client = depoc.DepocClient()


@click.group
def contact() -> None:
    ''' Contacts - retrieve all and filter. '''
    pass

@contact.command
@click.option('-l', '--limit', default=50)
@click.option('-p', '--page', default=0)
@click.option('-d', '--detailed', is_flag=True)
def all(limit: int, page: int, detailed: bool) -> None:
    ''' Retrieve all contacts. '''
    service = client.contacts.all

    if response := _handle_response(service, limit=limit, page=page):
        click.echo(f'\nResults: {response.count}')
        if limit < response.count:
            click.echo(
                f'Showing: {len(response.results)} out of {response.count}'
            ) 
        if response.next:
            click.echo(f'For next page: --page <number>')

        for obj in response.results:
            if hasattr(obj, 'customer'):
                title = f'{obj.customer.name}'
                header = 'customer'
                highlight = obj.customer.alias
                obj = obj.customer
            elif hasattr(obj, 'supplier'):
                title = f'\n{obj.supplier.legal_name}'
                header = 'supplier'
                highlight = obj.supplier.trade_name
                obj = obj.supplier
            
            remove = [] if detailed else \
            [item for item in obj.to_dict().keys() if item != 'id']

            _format_response(obj, title, header, highlight, remove=remove)


@contact.command
@click.option('-s', '--search')
@click.option('-l', '--limit', default=50)
@click.option('-d', '--detailed', is_flag=True)
def filter(search: str, limit: int, detailed: bool) -> None:
    ''' Filter contacts. '''
    service = client.contacts.filter

    if response := _handle_response(service, search=search, limit=limit):
        click.echo(f'\nResults: {response.count}')
        if limit < response.count:
            click.echo(
                f'Showing: {len(response.results)} out of {response.count}'
            ) 
        if response.next:
            click.echo(f'For next page: --page <number>')

        for obj in response.results:
            if hasattr(obj, 'customer'):
                title = f'\n{obj.customer.name}'
                header = 'customer'
                highlight = obj.customer.alias
                obj = obj.customer
            elif hasattr(obj, 'supplier'):
                title = f'\n{obj.supplier.legal_name}'
                header = 'supplier'
                highlight = obj.supplier.trade_name
                obj = obj.supplier
            
            remove = [] if detailed else \
            [item for item in obj.to_dict().keys() if item != 'id']
            
            _format_response(obj, title, header, highlight, remove=remove)



import depoc
import click

from .utils._response import _handle_response


client = depoc.DepocClient()


@click.command
@click.option('-d', '--date', default='week')
@click.option('-sd', '--start-date')
@click.option('-ed', '--end-date')
@click.pass_context
def report(ctx, date: str, start_date: str, end_date: str) -> None:
    ''' Financial report for a specific period. '''
    division = f'{'':->50}'

    if start_date and end_date:
        date = ''
        click.echo(f'{start_date +  ' - ' + end_date:>50}')
    else:
        click.echo(f'\n{date.upper():>50}')

    current_balance: float = 0
    total_receivable: float = 0
    total_payable: float = 0
    
    banks = client.financial_accounts.all
    receivables = client.receivables.filter
    payables = client.payables.filter

    if banks_response := _handle_response(banks):
        for obj in banks_response.results:
            current_balance += float(obj.balance)
        click.echo(division)
        format_current_balance = f'R$ {current_balance:.2f}'
        txt = f'BALANCE{format_current_balance:>43}'
        click.echo(txt)
        click.echo(division)

    if receivables_response := _handle_response(
        receivables,
        date=date,
        start_date=start_date,
        end_date=end_date,
    ):  
        for obj in receivables_response.results:
            total_receivable += float(obj.outstanding_balance)
        format_total_receivable = f'R$ {total_receivable:.2f}'
        txt = f'RECEIVABLE{format_total_receivable:>40}'
        click.echo(txt)
        click.echo(division)

    if payables_response := _handle_response(
        payables,
        date=date,
        start_date=start_date,
        end_date=end_date,
    ):
        for obj in payables_response.results:
            total_payable += float(obj.outstanding_balance)
        format_total_payable = f'R$ {total_payable:.2f}'
        txt = f'PAYABLE{format_total_payable:>43}'
        click.echo(txt)

        total_balance = round(
            current_balance + total_receivable - total_payable, 2
        )
        format_total_balance = f'R$ {total_balance:.2f}'
        color = 'red' if total_balance < 0 else 'green'

        click.echo(click.style(division, fg=color, bold=True))
        
        txt = click.style(
            f'RESULT{format_total_balance:>44}',
            fg=color,
            bold=True
        )
        click.echo(txt)

        click.echo(click.style(division, fg=color, bold=True))



import depoc
import click

from ..utils._response import _handle_response
from ..utils._format import _format_response


client = depoc.DepocClient()


@click.group
def bank() -> None:
    ''' Manage bank accounts '''
    pass

@bank.command
@click.argument('name')
def create(name: str) -> None:
    ''' Create a new bank account. '''
    data = {'name': name}
    service = client.financial_accounts.create

    if obj := _handle_response(service, data):
            highlight = f'R$ {obj.balance}'
            _format_response(obj, obj.name, highlight)

@bank.command
@click.argument('id')
def get(id: str) -> None:
    ''' Retrieve an specific bank account. '''
    service = client.financial_accounts.get

    if obj := _handle_response(service, resource_id=id):
        highlight = f'R$ {obj.balance}'
        _format_response(obj, obj.name, highlight)

@bank.command
def all() -> None:
    ''' Retrieve all bank accounts. '''
    service = client.financial_accounts.all

    total_balance: float = 0

    if response := _handle_response(service):
        for obj in response.results:
            total_balance += float(obj.balance)
            highlight = f'R$ {obj.balance}'
            remove = ['name', 'balance', 'created_at', 'is_active']
            _format_response(obj, obj.name, highlight, remove=remove)

        format_total_balance = f'R$ {total_balance:.2f}'
        txt = f'\n{'Total Balance: ' + format_total_balance:>50}\n'
        click.echo(txt)

@bank.command
@click.argument('id')
@click.argument('name')
def update(id: str, name: str) -> None:
    ''' Update a bank account. '''
    data = {'name': name}
    service = client.financial_accounts.update

    if obj := _handle_response(service, data, id):
            highlight = f'R$ {obj.balance}'
            _format_response(obj, obj.name, highlight)

@bank.command
@click.argument('id')
def delete(id: str) -> None:
    ''' Delete a bank account. '''
    service = client.financial_accounts.delete

    if obj := _handle_response(service, resource_id=id):
        _format_response(obj, 'DEACTIVATED', 'Done', color='red')



import depoc
import click

from .utils._response import _handle_response


client = depoc.DepocClient()


@click.command
@click.option('-d', '--date', default='week')
@click.option('-sd', '--start-date')
@click.option('-ed', '--end-date')
@click.pass_context
def balance(ctx, date: str, start_date: str, end_date: str) -> None:
    ''' Balance for a specific period. '''
    division = f'{'':->50}'

    if start_date and end_date:
        date = ''
        click.echo(f'{start_date +  ' - ' + end_date:>50}')
    else:
        click.echo(f'\n{date.upper():>50}')


    income: float = 0
    expenses: float = 0
    balance: float = 0
    
    service = client.financial_transactions.filter

    if response := _handle_response(
        service,
        date=date,
        start_date=start_date,
        end_date=end_date,
        ):
        for obj in response.results:
            if obj.type == 'credit':
                income += float(obj.amount)
            elif obj.type == 'debit':
                expenses += float(obj.amount)

        click.echo(division)
        format_income = f'R$ {income:.2f}'
        txt = f'INCOME{format_income:>44}'
        click.echo(txt)
        click.echo(division)

        format_expenses = f'R$ {expenses:.2f}'
        txt = f'EXPENSES{format_expenses:>42}'
        click.echo(txt)

        balance = round(income + expenses, 2)
        color = 'red' if balance < 0 else 'green'
        
        format_balance = f'R$ {balance:.2f}'

        click.echo(click.style(division, fg=color, bold=True))
        
        txt = click.style(
            f'BALANCE{format_balance:>43}',
            fg=color,
            bold=True
        )
        click.echo(txt)

        click.echo(click.style(division, fg=color, bold=True))
